'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

var _urljoin = require('urljoin');

var _urljoin2 = _interopRequireDefault(_urljoin);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

// Copyright 2016 Hound Technology, Inc. All rights reserved.
// Use of this source code is governed by the Apache License 2.0
// license that can be found in the LICENSE file.

/* global require, window, global */

/**
 * @module
 */
var superagent = require('superagent');


var userAgent = "libhoney-js/1.0.0-beta.4";

var _global = typeof window !== "undefined" ? window : typeof global !== "undefined" ? global : undefined;

// how many events to collect in a batch
var batchSizeTrigger = 50; // either when the eventQueue is > this length
var batchTimeTrigger = 100; // or it's been more than this many ms since the first push

// how many batches to maintain in parallel
var maxConcurrentBatches = 10;

// how many events to queue up for busy batches before we start dropping
var pendingWorkCapacity = 10000;

var emptyResponseCallback = function emptyResponseCallback() {};

var eachPromise = function eachPromise(arr, iteratorFn) {
  return arr.reduce(function (p, item) {
    return p.then(function () {
      return iteratorFn(item);
    });
  }, Promise.resolve());
};

/**
 * @private
 */

var Transmission = function () {
  function Transmission(options) {
    _classCallCheck(this, Transmission);

    this._responseCallback = emptyResponseCallback;
    this._batchSizeTrigger = batchSizeTrigger;
    this._batchTimeTrigger = batchTimeTrigger;
    this._maxConcurrentBatches = maxConcurrentBatches;
    this._pendingWorkCapacity = pendingWorkCapacity;
    this._sendTimeoutId = -1;
    this._eventQueue = [];
    this._batchCount = 0;
    // Included for testing; to stub out randomness and verify that an event
    // was dropped.
    this._randomFn = Math.random;
    this._droppedCallback = emptyResponseCallback;

    if (typeof options.responseCallback == "function") {
      this._responseCallback = options.responseCallback;
    }
    if (typeof options.batchSizeTrigger == "number") {
      this._batchSizeTrigger = options.batchSizeTrigger;
    }
    if (typeof options.batchTimeTrigger == "number") {
      this._batchTimeTrigger = options.batchTimeTrigger;
    }
    if (typeof options.maxConcurrentBatches == "number") {
      this._maxConcurrentBatches = options.maxConcurrentBatches;
    }
    if (typeof options.pendingWorkCapacity == "number") {
      this._pendingWorkCapacity = options.pendingWorkCapacity;
    }
  }

  _createClass(Transmission, [{
    key: 'sendEvent',
    value: function sendEvent(ev) {
      // bail early if we aren't sampling this event
      if (!this._shouldSendEvent(ev)) {
        this._droppedCallback();
        return;
      }

      this._eventQueue.push(ev);
      if (this._eventQueue.length > this._batchSizeTrigger) {
        this._sendBatch();
      } else {
        this._ensureSendTimeout();
      }
    }
  }, {
    key: '_sendBatch',
    value: function _sendBatch() {
      var _this = this;

      if (this._batchCount == maxConcurrentBatches) {
        // don't start up another concurrent batch.  the next timeout/sendEvent or batch completion
        // will cause us to send another
        return;
      }

      this._clearSendTimeout();

      this._batchCount++;

      var batch = this._eventQueue.splice(0, this._batchSizeTrigger);

      eachPromise(batch, function (ev) {
        var url = (0, _urljoin2.default)(ev.apiHost, "/1/events", ev.dataset);
        var req = superagent.post(url);

        return new Promise(function (resolve) {
          req.set('X-Hny-Team', ev.writeKey).set('X-Hny-Samplerate', ev.sampleRate).set('X-Hny-Event-Time', ev.timestamp.toISOString()).set('User-Agent', userAgent).type("json").send(ev.postData).end(function (err, res) {
            // call a callback here (in our init options) so it can be used both in the node, browser, and worker contexts.
            _this._responseCallback({ err: err, res: res });

            // we resolve unconditionally to continue the iteration in eachSeries.  errors will cause
            // the event to be re-enqueued/dropped.
            resolve();
          });
        });
      }).then(function () {
        _this._batch--;
        if (_this._eventQueue.length > _this._batchSizeTrigger) {
          _this._sendBatch();
        }
      }).catch(function () {
        _this._batch--;
        if (_this._eventQueue.length > _this._batchSizeTrigger) {
          _this._sendBatch();
        }
      });
    }
  }, {
    key: '_shouldSendEvent',
    value: function _shouldSendEvent(ev) {
      if (this._eventQueue.length >= this._pendingWorkCapacity) {
        return false;
      }
      var sampleRate = ev.sampleRate;

      if (sampleRate <= 1) {
        return true;
      }
      return this._randomFn() < 1 / sampleRate;
    }
  }, {
    key: '_ensureSendTimeout',
    value: function _ensureSendTimeout() {
      var _this2 = this;

      if (this._sendTimeoutId === -1) {
        this._sendTimeoutId = _global.setTimeout(function () {
          return _this2._sendBatch();
        }, this._batchTimeTrigger);
      }
    }
  }, {
    key: '_clearSendTimeout',
    value: function _clearSendTimeout() {
      if (this._sendTimeoutId !== -1) {
        _global.clearTimeout(this._sendTimeoutId);
        this._sendTimeoutId = -1;
      }
    }
  }]);

  return Transmission;
}();

exports.default = Transmission;